%CAL_READNOISE calculates the read noise/gain of a CCD
%
% SYNOPSIS:
%  out = cal_readnoise(in, bg, N, rg, sh)
%
% PARAMETERS:
%  in: input image (a stack of images, containing repeated measurements 
%      of the same object); best if all possible intensities are present
%      in the image (create a intensity gradient, e.g. out-of-focus edge) 
%  bg: background image
%  N:  number of intensity bins (default 100) x coord is center of a bin
%  rg: fit range, e.g. [1e3 1e4] (default -1, the whole range of intensities)
%  sh: perform shift alignment before evaluation
%
%  out(1) n(e-) sqrt(of y-axis intersection) / slope
%  out(2) the slope of the fit: conversion ADU = detected #photons * slope
%
%  The gain is obtained by fitting a line to the variance vs intensity plot
%  either weighted or unweighted dependent on which toolboxes are avaibable
%
% LITERATURE:
%  L.J. van Vliet, D. Sudar and I.T. Young in Cell Biology volume III, J.E. Celis (eds.)
%  Digital Fluorescence Imaging Using Cooled CCD Array Cameras, pp.109-120, 1998

% (C) Copyright 2004           Department of Molecular Biology
%     All rights reserved      Max-Planck-Institute for Biophysical Chemistry
%                              Am Fassberg 11, 37077 G"ottingen
%                              Germany
%
% Bernd Rieger, Rainer Heintzmann August 2004.
% added shift correction (Dec 2004)
% bin(0)=bg and weighted fit (BR, KL Dec 2004)

function out = cal_readnoise(varargin)

d = struct('menu','Analysis',...
   'display','Read noise calibration of a CCD',...
   'inparams',struct('name',       {'a','abg','N','rg','sh'},...
         'description',{'Input image','Background image',...
            'Number of intensity bins','fit range [ADU]','Shift correction'},...
         'type',       {'image','image','array','array','boolean'},...
         'dim_check',  {0,0,0,-1,0},...
         'range_check',{[],[],'N+','R',[]},...
         'required',   {1,1,0,0,0},...
         'default',    {'a','b',100,-1,0}...
        ),...
   'outparams',struct('name',{'out'},...
                      'description',{'Output image'},...
                      'type',{'image'}...
                      )...
  );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [a,abg,N,rg,sh] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end
if ndims(a)~=3
	error('Input stack should be 3D, series of same measuremetns.');
end
if sh
   [tmp,s]=correctshift(a);
   m=max(max(abs(s)));
   sz=size(a{1});
   a = cut(tmp,[sz(1:2)-ceil(m) sz(3)]);
   fprintf(' Recropping the image to size %d %d %d\n',size(a))
   clear tmp
end
bg = mean(abg);
t = a-bg;
clear a 
no = sum(t(:,:,0));

for ii=1:size(t,3)-1
   t(:,:,ii) = t(:,:,ii)* no/sum(t(:,:,ii));
end

mz = squeeze(mean(t,[],3));    %mean in ADU
v =  squeeze(var(t,[],3));     %variance in ADU

N = N-1; %this leads to N bins
rmz = dip_image(floor(mz/max(mz)*N+.5)+1,'sint32');%quantize the ADU into N bins, use this as label for measure
msr = measure(rmz,v,{'Mean'},[],1,0,0);
tp1 =[msr.ID] .*max(mz)/N;
x = [0 tp1];
y = [double(var(abg)) msr.Mean];

h=figure;
plot(x,y,'kx')
xlabel('Intensity [ADU]','FontSize',18)
ylabel('Variance [ADU^2]','FontSize',18)
ax = axis;
axis([0 ax(2) 0 ax(4)])
g=gca;
set(g,'FontSize',12)

if exist('fit')
   fprintf(' Weigthed fit\n');
   %get the weights (number of pixel per bin)
   msr2 = measure(rmz,rmz,{'Sum'},[],1,0,0);
   w1 = msr2.Sum./msr2.id;
   szbg = size(abg);
   sz3 = size(t,3);
   w = [prod(szbg) w1.*sz3]; %number of pixel that are in that bin
   w = w./sum(w);

   opts = fitoptions;
   if rg<0   
      opts = fitoptions(opts,'Weights',w);
      f = fit(x',y','poly1',opts);   
   else
      mask = x > rg(1) & x <rg(2);
      opts = fitoptions(opts,'Weights',w(mask));
      f = fit(x(mask)',y(mask)','poly1',opts);   
   end
   out(1) = f.p2;
   out(2) = f.p1;   
   hold on
   plot(x,x*out(2)+out(1),'r');
   hold off
   
   figure;%plot weigths also
   plot(x,y,'kx');hold on
   plot(x,x*out(2)+out(1),'r');hold off
   xlabel('Intensity [ADU]','FontSize',18)
   ylabel('Variance [ADU^2]','FontSize',18)
   ax = axis;
   axis([0 ax(2) 0 ax(4)]);
   ax1=gca;
   set(ax1,'FontSize',12)   
   ax2 = axes('Position',get(ax1,'Position'),...
      'XAxisLocation','top','YAxisLocation','right',...
      'Color','none','XColor','b','YColor','b');   
   ylabel('relative weight','FontSize',16,'Parent',ax2)   
   if rg<0
      line(x,w);
   else
      line(x(mask),w(mask));
   end  
   axis(ax2,[0 ax(2) 0 .5])
   set(ax2,'YScale','log');
   
   hold off
elseif exist('robustfit')
   fprintf(' robust fit (unweigthed)\n');
   if rg<0
      [out,stats]=robustfit(x,y);
   else
      mask = x > rg(1) & x <rg(2);
      [out,stats]=robustfit(x(mask),y(mask));
   end
   hold on
   plot(x,x*out(2)+out(1),'r');
   hold off
else
   warning('No fitting functionality found.');
   out = -1;
   return   
end
figure(h)

out(1) = y(1);%readnoise given by variation in background image 
out(1) = sqrt(out(1))/out(2);

s=sprintf('gain: %0.2g,\nreadnoise: %0.2g e^-',...
   out(2),out(1));
text(x(4),x(end-10)*out(2)+out(1),s,'FontSize',14)

