%DIPSTACKINSPECT  Interactive inspection of the thrid dimension  
%   [out,p] = DIPSTACKINSPECT(H,CW) for a 3D figure window with handle H.
%
% Left-click in the image to see the intensity along the hidden dimension.
% Right-click to terminate.
%
% OUTPUT:
%   out: 1D image along the selected point
%   p:   Coordinate of the selected point
%
% PARAMETERS:
%   H:   figure handle
%   CW:  boolean, close window after terimnation?
%
%
%   It is still possible to use all the menus in the victim figure
%   window, but you won't be able to access any of the tools (like
%   zooming and testing). You can, however, step through a 3D
%   image by using the keyboard.
%
%   Note: If you feel the need to interrupt this function with Ctrl-C,
%   you will need to refresh the display (by re-displaying the image
%   or changing the 'Actions' state).
%
%   See also DIPSHOW, DIPTEST, DIPCROP, DIPPROFILE, DIPGETCOORDS.

% (C) Copyright 1999-2006               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Bernd Rieger October 2006
% 27 October 2006: Fixed bug: the second call to this function always failed.
%                  Made figure handle input argument optional. (CL)

function [out,coords] = dipstackinspect(varargin)

d = struct('menu','Display',...
      'display','Stack inspeck',...
      'inparams',struct('name',       {'fig','clwin'},...
                        'description',{'Figure window','Close window after termination?'},...
                        'type',       {'handle','boolean'},...
                        'dim_check',  {0,0},...
                        'range_check',{'3D',''},...
                        'required',   {0,0},...
                        'default',    {[],0}...
                       ),...
      'outparams',struct('name',{'out','coords'},...
                         'description',{'z-Intensity','Coordinate array'},...
                         'type',{'image','array'}...
                         )...
     );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [fig,clwin] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end


tag = get(fig,'Tag');
if ~strncmp(tag,'DIP_Image_3D',12)
   error('DIPSTACKINSPECT only works on 3D images displayed using DIPSHOW.')
end
ax = findobj(fig,'Type','axes');
if length(ax)~=1
   error('DIPSTACKINSPECT only works on images displayed using DIPSHOW.')
end

% Store old settings
au = get(ax,'Units');
wbdF = get(fig,'WindowButtonDownFcn');
wbuP = get(fig,'WindowButtonUpFcn');
wbmF = get(fig,'WindowButtonMotionFcn');
bdF = get(fig,'ButtonDownFcn');
pscd = get(fig,'PointerShapeCData');
pshs = get(fig,'PointerShapeHotSpot');
ptr = get(fig,'pointer');

% Set new settings
figure(fig);
set(fig,'WindowButtonDownFcn','',...
        'WindowButtonUpFcn','',...
        'WindowButtonMotionFcn','',...
        'ButtonDownFcn','');
dipfig_setpointer(fig,'cross');
set(ax,'Units','pixels');

% Do your stuff
h = figure;
h = axes('parent',h);
pt = [];
imline = [];
done = 0;
while ~done
   while 1
      set(fig,'WindowButtonDownFcn','set(gcbf,''WindowButtonDownFcn'',''Click!'')');
      waitfor(fig,'WindowButtonDownFcn'); % The ButtonDown callback changes the callback.
                                          % This way, we also detect a change in state!
      if ~ishandle(fig)
         error('You closed the window! That wasn''t the deal!')
         done = 1;
         break;
      end
      if ~strcmp(get(fig,'WindowButtonDownFcn'),'Click!')
         % The user just changed the state. Store the new settings and revert to our own...
         wbdF = get(fig,'WindowButtonDownFcn');
         wbuP = get(fig,'WindowButtonUpFcn');
         wbmF = get(fig,'WindowButtonMotionFcn');
         bdF = get(fig,'ButtonDownFcn');
         pscd = get(fig,'PointerShapeCData');
         pshs = get(fig,'PointerShapeHotSpot');
         ptr = get(fig,'pointer');
         set(fig,'WindowButtonDownFcn','',...
                 'WindowButtonUpFcn','',...
                 'WindowButtonMotionFcn','',...
                 'ButtonDownFcn','');
         dipfig_setpointer(fig,'cross');
         set(ax,'Units','pixels');
         continue;
      else
         switch get(fig,'SelectionType');
         case 'normal' % left click
            pt = dipfig_getcurpos(ax);
         case 'alt'    % right click
            done = 1;
         end
         break;
      end
   end
   if done
      break
   end
   udata = get(fig,'UserData');
   switch udata.slicing
      case 'yz'
         udata = get(fig,'UserData');
         pt = [udata.curslice,pt];
         imline = squeeze(double(udata.slices(:,pt(2),pt(3))));
         ttl = [':,',num2str(pt(2)),',',num2str(pt(3))];
      case 'xz'
         udata = get(fig,'UserData');
         pt = [pt(1),udata.curslice,pt(2)];
         imline = squeeze(double(udata.slices(pt(1),:,pt(3))));
         ttl = [num2str(pt(1)),',:,',num2str(pt(3))];
      otherwise % case 'xy'
         udata = get(fig,'UserData');
         pt = [pt,udata.curslice];
         imline = squeeze(double(udata.slices(pt(1),pt(2),:) ));
         ttl = [num2str(pt(1)),',',num2str(pt(2)),',:'];
   end
   if ~ishandle(h)
	   h = figure;
	   h = axes('parent',h);
   end
   plot(h,[0:length(imline)-1],imline);
   xlabel(h,'Stack index');
   ylabel(h,'Intensity');
   title(h,['Intensity at (',ttl,')']);
end

% Restore old settings
set(ax,'Units',au);
set(fig,'WindowButtonDownFcn',wbdF,...
        'WindowButtonUpFcn',wbuP,...
        'WindowButtonMotionFcn',wbmF,...
        'ButtonDownFcn',bdF,...
        'PointerShapeCData',pscd,...
        'PointerShapeHotSpot',pshs,...
        'pointer',ptr);

if clwin
   close(get(h,'parent'));
end

% Generate output
if isempty(imline)
   error('No point selected');
else
   if nargout~=0
      out = dip_image(imline);
      coords = pt;
   end
end
