%GAUSS_DERIVATIVE   Gaussian derivatives
%
% SYNOPSIS:
%  image_out = gauss_derivative(image_in,sigma,order)
%
% PARAMETERS:
%  sigma = smoothing for the different directions
%          sigma=0 and order=0: these directions not processed
%          sigma=0 and order>0: fourier derivative
%  order:  derivative order
%          order>3:             fourier derivative
%
% DEFAULTS:
%  sigma = 1
%  order = 1
%
% EXAMPLE:
%  a = readim;
%  gauss_derivative(a,1,0) - gaussf(a,1)     %only border and roundoff difference
%  gauss_derivative(a,[1 1],[1 0])-dx(a,1)   %identical
%  gauss_derivative(a,[1 0],[1 0])-dx(a,[1 0]) %identical
%  gauss_derivative(a,[1 0],[1 0])-dx(a,1)   %no blurring in y direction!
%                                            %compare dip_gauss and the process array
%
% NOTES:
%  The method used to compute the Gaussian derivative is chosen through the
%  'DerviativeFlavour' preference (see DIPSETPREF). These are the possible
%  values:
%    - 'fir' :     Finite Impulse Resonse filter (convolution with a kernel).
%    - 'iir' :     Infinte Impulse Response filter (recursive filtering).
%    - 'fourier' : Convolution via a multiplication in the Fourier Domain.
%    - 'default' : Chooses the best option above for your kernel. This is
%                  the default.
%
%  The "best option for your kernel" means the following: The FIR method is
%  always used except if any sigma is smaller than 0.8, in which case the
%  Fourier method is used since the FIR method does not work for small kernel
%  sizes. If any sigma equals 0 but the corresponding order is also 0, this
%  dimension is ignored, and thus does not influence the method chosen.
%
%  The preference above is overridden if:
%   - any ORDER is larger than 3
%   - any sigma is 0 and the corresponding order is not
%  In these cases, the Fourier method is always used.
%
%  See the 'Truncation' preference for changing the size of the kernel used
%  by the FIR method.
%
% SEE ALSO:
%  dip_gauss, dip_gaussiir, dipsetpref('DerviativeFlavour'), dipsetpref('Truncation')

% (C) Copyright 1999-2006               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Lucas van Vliet, May 2000.
% April 2004:  Derivative of higher orders are now computed in the Fourier domain! (KvW)
% August 2004: Derivatives >3 via ft instead >2 (BR)
% June 2006:   Integrated the code from GAUSS_FOURIER into here.
% October 2006: Added possibility to always do Fourier domain implemenation via preference 
%               and always do fourier for sigma <1. All dx,dy etc now are only a front-end
%               to this function, which should make any future changes easier (BR).
% November 2006: changed behaviour of sigma=0 and order=0 as CL suggested (BR)
% 8 November 2006: Changed behaviour for DerivativeFlavour. (CL)

function out = gauss_derivative(varargin)

d = struct('menu','Differential Filters',...
           'display','General Gaussian derivatives',...
           'inparams',struct('name',       {'image_in',   'sigma',            'order'},...
                             'description',{'Input image','Sigma of gaussian','Derivative Order'},...
                             'type',       {'image',      'array',            'array'},...
                             'dim_check',  {0,            1,                  1},...
                             'range_check',{[],           'R+',               'N'},...
                             'required',   {1,            0,                  0},...
                             'default',    {'a',          1,                  1}...
                            ),...
           'outparams',struct('name',{'out'},...
                              'description',{'Output image'},...
                              'type',{'image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [image_in,sigma,order] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

if any(order>3) || any(order>0 & sigma==0)
   df = 2; % Always use Fourier method in these cases.
else
   df = lower(dipgetpref('DerivativeFlavour'));
   switch df
      case 'fourier'
         df = 2;
      case {'fir','spatial'}
         df = 0;
      case 'iir'
         df = 1;
      case {'best','default'}
         if any(sigma>0 & sigma<0.8)
            df = 2;
         else
            % At some point we'll make IIR the default for large images or something...
            df = 0;
         end
      otherwise
         error('Illegal value for DerivativeFlavour preference (see DIPSETPREF)')
   end
end

switch df
   case 0 % FIR
      process = ones(size(sigma));
      process(sigma<=0) = 0;
      out = dip_gauss(image_in,process,sigma,order); 
   case 1 % IIR
      process = ones(size(sigma));
      process(sigma<=0) = 0;
      filterorder = repmat(3,size(sigma));
      % default design method: discrete time fit
      % 1=forward backward method
      % 2=discrete time fit
      out = dip_gaussiir(image_in,process,sigma,order,filterorder,2);
   case 2 % Fourier
      % We need to implement this in DIPlib!!!
      disp('Computing a Gaussian through the FT!!!')
      inft = ft(image_in);
      for ii=1:ndims(image_in)
         if order(ii)==0 & sigma(ii)==0
            % Nothing happens...
            continue;
         end
         t = createramp(size(image_in),ii,'freq');
         inft = inft .* ( (i*2*pi) * t ).^order(ii) .* exp( -0.5 * ( (2*pi*sigma(ii)) * t ).^2 );
      end
      out = ift(inft);
      if isreal(image_in);
         out = real(out);
      end
end
