%GAUSSF   Gaussian filter
%
% SYNOPSIS:
%  image_out = gaussf(image_in,sigma)
%
% DEFAULTS:
%  sigma = 1
%
% NOTES:
%  The method used to compute the Gaussian derivative is chosed through the
%  'DerviativeFlavour' preference (see DIPSETPREF). These are the possible
%  values:
%    - 'fir' :     Finite Impulse Resonse filter (convolution with a kernel).
%    - 'iir' :     Infinte Impulse Response filter (recursive filtering).
%    - 'fourier' : Convolution via a multiplication in the Fourier Domain.
%    - 'default' : Chooses the best option above for your kernel. This is
%                  the default.
%
%  The "best option for your kernel" means the following: The FIR method is
%  always used except if any sigma is smaller than 0.8, in which case the
%  Fourier method is used since the FIR method does not work for small kernel
%  sizes. If any sigma equals 0, this dimension is ignored, and thus does not
%  influence the method chosen.
%
%  See the 'Truncation' preference for changing the size of the kernel used
%  by the FIR method.
%
% SEE ALSO:
%  dip_gauss, dip_gaussiir, dipsetpref('DerviativeFlavour'), dipsetpref('Truncation')

% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Lucas van Vliet, May 2000.
% added fourier implementation if sigma <1 or DerivativeFlavour is set to fourier (BR)
% 8 November 2006: Calling GAUSS_DERIVATIVE now. (CL)

function image_out = gaussf(varargin)

if nargin==0
   if exist('private/Gauss.jpg','file')
      image_out = dip_image(imread('private/Gauss.jpg'));
      return
   end
end

d = struct('menu','Filters',...
           'display','Gaussian filter',...
           'inparams',struct('name',       {'image_in',   'sigma'},...
                             'description',{'Input image','Sigma of gaussian'},...
                             'type',       {'image',      'array'},...
                             'dim_check',  {0,            1},...
                             'range_check',{[],           'R+'},...
                             'required',   {1,            0},...
                             'default',    {'a',          1}...
                            ),...
           'outparams',struct('name',{'image_out'},...
                              'description',{'Output image'},...
                              'type',{'image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      image_out = d;
      return
   end
end
try
   [image_in,sigma] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

order = zeros(size(sigma));
image_out = gauss_derivative(image_in,sigma,order);
