%RADONCIRCLE compute Radon/Hough transform to find circles/spheres
%
% SYNOPSIS:
%  [rt, p, o] = radoncircle(in, radii, sigma)
%
% OUTPUT: 
%  rt:  Radon transform space
%  p:   Parameters of the found circles as (N, [x y r])
%  o:   Overlay image of input with found circles
%
% PARAMETERS:
%  in:
%  radii:  range of radii to search, e.g. [30:5:45]
%          use more radii for higher sensitivity.
%  sigma:  width of the structures 
%  minval: minimal height difference between peak and valley
%          in the Radon image; smaller is more sensitive.
%
% DEFAULTS:
%  radii = [10:20]
%  sigma = 1 
%  minval = []
%
% NOTE:
%  Works for nD, except overlay ouput. 
%
% EXAMPLE:
%  a = gaussianlineclip(rr-30) + gaussianlineclip(rr-50); 
%  b = a + shift(a,[10,-30])
%  [rt,p,o] =  radoncircle(b, [25:2:55])
%
% LITERATURE:
%  C.L. Luengo Hendriks, M. van Ginkel, P.W. Verbeek and L.J. van Vliet,
%   The Generalized Radon Transform: Sampling, Accuracy and Memory Considerations,
%   Pattern Recognition, 38(12):2494-2505, 2005.

% (C) Copyright 1999-2006               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Bernd Rieger, Oct 2006 (based on code by CL and MvG).

function varargout = radoncircle(varargin)

d = struct('menu','Segmentation',...
           'display','Radon Transform (Circle/Sphere)',...
           'inparams',struct('name',       {'in',   'radii','sigma','minvalue'},...
                             'description',{'Input image','Radius range [r0:r1]','Sigma','Height difference'},...
                             'type',       {'image',      'array','array','array'},...
                             'dim_check',  {0,            -1,0,-1},...
                             'range_check',{[],           'R+','R+','R+'},...
                             'required',   {1,            0,0,0},...
                             'default',    {'a',          [10:20],1,[]}...
                            ),...
           'outparams',struct('name',{'out_radon','para','im_overlay'},...
      'description',{'Radon transform','Circle(s) parameters [r x y (z)]','Overlay'},...
                              'type',{'image','array','image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end
try
   [in,radii,sigma,minvalue] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

sigma=1;
out=newim([size(in) length(radii)],'sfloat');

ftin=ft(in);
jj=0;
%#function real ift
ss = ['out(' repmat(':,',1,ndims(in)) 'jj)=real(ift(ftin*ftdisk));']; % to make it 2/3D
for ii=radii
   ftdisk=ft(gaussianlineclip(rr(in)-ii,sigma));
   %out(:,:,jj)=real(ift(ftin*ftdisk));
   eval(ss);
   jj=jj+1;
end
varargout{1}=out;
if nargout <2; return;end

m = findmaxima(out,minvalue,'joint')
if isempty(m)
   warning('Could not find any good maxima in the Radon Transform.');
   [varargout{:}]=[];
   varargout{1} = out;
   return;
end
for ii=1:size(m,1)
   r(ii) = get_subpixel(radii,m(ii,end));   
end
out2 = m;
out2(:,end) = r;
varargout{2} = out2;
if nargout<3;return;end

lab = newim(in,'uint8');
sz = size(in);
for ii=1:size(m,1)
   if ndims(in)==2
      c = (xx(sz,'corner')-out2(ii,1))^2+(yy(sz,'corner')-out2(ii,2))^2< r(ii)^2;
   else
      c =(xx(sz,'corner')-out2(ii,1))^2+(yy(sz,'corner')-out2(ii,2))^2+(zz(sz,'corner')-out2(ii,3))^2< r(ii)^2;
   end
   c = xor(c,bdilation(c));
   lab(c) = ii;
end
varargout{3} = overlay(stretch(in),lab);
